"""
This package has no tests, but for the automatic deployment
to occur, tests need to run and pass. If you can think of a
useful test to add to the package, please do so, and then this
module can be removed.
"""

def test_null():
	"A trivial test to enable the test runner to pass"
